#include <stdio.h>
#include <errno.h>
#include <sys/mman.h>
#include <hbwmalloc.h>

#define PAGESIZE 0x200000
//#define PAGESIZE 0x1000

#if PAGESIZE == 0x200000
#define PAGESIZE_FLAG HBW_PAGESIZE_2MB
#elif PAGESIZE == 0x1000
#define PAGESIZE_FLAG HBW_PAGESIZE_4KB
#endif

int alloc_a_off(void ** memptr, size_t nelm)
{
  void * ptr;
  char * poke;
  size_t size;
  size_t span;
  int retval; 

  printf("Allocating a_off, PAGESIZE=0x%x\n", PAGESIZE);

  size = nelm*sizeof(float);
  span = (size + PAGESIZE-1) & ~(PAGESIZE-1);

  retval = hbw_posix_memalign_psize(&ptr, PAGESIZE, nelm*sizeof(float), PAGESIZE_FLAG);
  if (retval != 0) {
    char const * errstr = strerror(errno);
    printf("hbw_posix_memalign_psize failed: %s\n", errstr);
    return retval;
  }

  retval = mlock(ptr, span);
  if (retval != 0) {
    char const * errstr = strerror(errno);
    printf("mlock failed: %s\n", errstr);
    return retval;
  }

  retval = posix_madvise(ptr, span, POSIX_MADV_WILLNEED | POSIX_MADV_SEQUENTIAL);
  if (retval != 0) {
    char const * errstr = strerror(errno);
    printf("posix_madvise failed: %s\n", errstr);
    return retval;
  }

  poke = ptr;
  do {
    *poke = 0xf;
    poke += PAGESIZE;
  } while (poke < (ptr+span));

  *memptr = ptr;

  return 0;
}
